/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#ifndef _STM32_H_
#define _STM32_H_

#include "sim_generic_type_defs.h"
#include "stm32_defs.h"
#include "sim_debug.h"

#ifdef __cplusplus
extern "C"{
#endif

#define HIGH                    1
#define LOW                     0
#define INPUT                   0
#define OUTPUT                  1
#define INPUT_PULLUP            2
#define INPUT_PULLDOWN          3
#define OUTPUT_OPENDRAIN        4
#define INPUT_DISABLE           5
#define LSBFIRST                0
#define MSBFIRST                1
#define CHANGE                  4
#define FALLING                 2
#define RISING                  3

//
#define min(a,b) ((a)<(b)?(a):(b))
#define max(a,b) ((a)>(b)?(a):(b))
#define constrain(amt,low,high) ((amt)<(low)?(low):((amt)>(high)?(high):(amt)))
#define radians(deg) ((deg)*DEG_TO_RAD)
#define degrees(rad) ((rad)*RAD_TO_DEG)
#define sq(x) ((x)*(x))

#define lowByte(w) ((uint8_t) ((w) & 0xff))
#define highByte(w) ((uint8_t) ((w) >> 8))

#define bitRead(value, bit) (((value) >> (bit)) & 0x01)
#define bitSet(value, bit) ((value) |= (1UL << (bit)))
#define bitClear(value, bit) ((value) &= ~(1UL << (bit)))
#define bitWrite(value, bit, bitvalue) (bitvalue ? bitSet(value, bit) : bitClear(value, bit))
//user functions
void setup();
void loop();
void analogReadResolution(unsigned char);//set resolution for all ADC inputs
void analogWriteResolution(unsigned char);//set resolution for all PWM outputs
void analogReadResolutionPin(unsigned char, unsigned char);//set ADC resolution for a pin
void analogWriteResolutionPin(unsigned char, unsigned char);//set resolution for a PWM output pin
void pinMode(unsigned char pin, unsigned char mode);
void pinCore(unsigned char pin, unsigned char mode, unsigned char core_idx);
void digitalWrite(unsigned char, unsigned char);
int digitalRead(unsigned char);
int analogRead(unsigned char);
//void analogReference(unsigned char mode);
void analogWrite(unsigned char, int);
unsigned long long millis();
unsigned long long micros();
void delay(unsigned long long);
void delayMicroseconds(unsigned long long);
inline int digitalPinToInterrupt(unsigned int pin){return pin;}//should be inline
void attachInterrupt(int pin_idx, void (*function)(void), unsigned int mode, unsigned int pullup=0);
//internal functions
void sys_init();
void init1();
void init2();
void initVariant1();//executed before setup()
void initVariant2();//executed after setup(). useful in adc/pwm resolution settings
void loop_stm32(void);
TIM_TypeDef* pwmGetTimerFromPin(unsigned char);
GPIO_TypeDef* dioGetGPIOPortFromPin(unsigned char);
unsigned int dioGetGPIOPinFromPin(unsigned char);
//
void initADC();
void startADC();
void updateADC();
void initPWM();//initializes pwm & dac
void startPWM();
void initDIO();
///////////////////////////serial ports///////////////////////////////////
#define SERIAL_TX_BUFFER_SIZE	128
#define SERIAL_RX_BUFFER_SIZE	128
//
#define SERIAL_8N1				0x01
#define SERIAL_8E1				0x02
#define SERIAL_8O1				0x03
//
class SerialComm
{
public:
    SerialComm();
    ~SerialComm();
    void begin(unsigned int, _BYTE);
    void begin(unsigned long baud) { begin(baud, SERIAL_8N1); }
    void end();
    int available();
    int peek(int index = 0);
    int read();
    int readBytes(unsigned char*, int);
    inline int readBytes(char *buffer, int length) {return readBytes((unsigned char*)buffer, length);}
    void write(unsigned char);
    int write(unsigned char*, int);
    void flush();
private:
    volatile int m_read_index;
};
extern SerialComm Serial;
//
class SerialComm1
{
public:
    SerialComm1();
    ~SerialComm1();
    void begin(unsigned int, _BYTE);
    void begin(unsigned long baud) { begin(baud, SERIAL_8N1); }
    void end();
    int available();
    int peek(int index = 0);
    int read();
    int readBytes(unsigned char*, int);
    inline int readBytes(char *buffer, int length) {return readBytes((unsigned char*)buffer, length);}
    void write(unsigned char);
    int write(unsigned char*, int);
    void flush();
private:
    volatile int m_read_index;
};
extern SerialComm1 Serial1;
//
class SerialComm2
{
public:
    SerialComm2();
    ~SerialComm2();
    void begin(unsigned int, _BYTE);
    void begin(unsigned long baud) { begin(baud, SERIAL_8N1); }
    void end();
    int available();
    int peek(int index = 0);
    int read();
    int readBytes(unsigned char*, int);
    inline int readBytes(char *buffer, int length) {return readBytes((unsigned char*)buffer, length);}
    void write(unsigned char);
    int write(unsigned char*, int);
    void flush();
private:
    volatile int m_read_index;
};
extern SerialComm2 Serial2;
//
class SerialComm3
{
public:
    SerialComm3();
    ~SerialComm3();
    void begin(unsigned int, _BYTE);
    void begin(unsigned long baud) { begin(baud, SERIAL_8N1); }
    void end();
    int available();
    int peek(int index = 0);
    int read();
    int readBytes(unsigned char*, int);
    inline int readBytes(char *buffer, int length) {return readBytes((unsigned char*)buffer, length);}
    void write(unsigned char);
    int write(unsigned char*, int);
    void flush();
private:
    volatile int m_read_index;
};
extern SerialComm3 Serial3;
//
class SerialComm4
{
public:
    SerialComm4();
    ~SerialComm4();
    void begin(unsigned int, _BYTE);
    void begin(unsigned long baud) { begin(baud, SERIAL_8N1); }
    void end();
    int available();
    int peek(int index = 0);
    int read();
    int readBytes(unsigned char*, int);
    inline int readBytes(char *buffer, int length) {return readBytes((unsigned char*)buffer, length);}
    void write(unsigned char);
    int write(unsigned char*, int);
    void flush();
private:
    volatile int m_read_index;
};
extern SerialComm4 Serial4;
/////////////////////////////firmware helpers///////////////////////////////
void SetPortPinForGPOut(GPIO_TypeDef *port, _DWORD pin_number);
void SetPortPinForGPIn(GPIO_TypeDef *port, _DWORD pin_number);
void EnablePortPinPullup(GPIO_TypeDef *port, _DWORD pin_number);
void DisablePortPinPullup(GPIO_TypeDef *port, _DWORD pin_number);
void WritePortPinHigh(GPIO_TypeDef *port, _DWORD pin_number);
void WritePortPinLow(GPIO_TypeDef *port, _DWORD pin_number);
_DWORD ReadOutputPortPin(GPIO_TypeDef *port, _DWORD pin_number);
_DWORD ReadInputPortPin(GPIO_TypeDef *port, _DWORD pin_number);
void WritePort(GPIO_TypeDef *port, _DWORD data);
_DWORD ReadOutputPort(GPIO_TypeDef *port);
_DWORD ReadInputPort(GPIO_TypeDef *port);
////////////////////////////////////////////////////////////////////////////
//This function is executed in case of error occurrence
void _Error_Handler(const char *, int);
#define Error_Handler() _Error_Handler(__FILE__, __LINE__)
////////////////////////////////////////////////////////////////////////////
#ifdef __cplusplus
}
#endif
#endif //_STM32_H_
