/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#ifndef STM32_I2C_H
#define STM32_I2C_H

#include "stm32.h"

/**
 * @brief Initialze the I2Cx peripheral
 * @param i2c The I2Cx peripheral handle
 */
bool i2cInit();
void i2cDisable();
void i2cSetAddress(uint8_t address);//only in slave mode
void i2cSetFrequency(uint32_t clk);

/**
 * @brief Send bytes to a slave
 * @note 0 < @p length <= 255
 * @param i2c The I2Cx peripheral handle
 * @param slaveAddr The slave address (including the R/W bit)
 * @param data The buffer containing the bytes to be sent
 * @param length The length of the buffer (number of bytes)
 */
uint8_t i2cSend(uint8_t slaveAddr, uint8_t* data, uint8_t length, uint8_t send_stop = 1);

/**
 * @brief Send one byte to a slave
 * @param i2c The I2Cx peripheral handle
 * @param slaveAddr The slave address (including the R/W bit)
 * @param data The byte to be sent
 */
void i2cSendByte(uint8_t slaveAddr, uint8_t data);

/**
 * @brief Write a specific value to a specific register in a slave
 * @param i2c The I2Cx peripheral handle
 * @param slaveAddr The slave address (including the R/W bit)
 * @param regAddr The address of the register
 * @param value The value to be written to the specified register
 */
void i2cWriteReg(uint8_t slaveAddr, uint8_t regAddr, uint8_t value);

/**
 * @brief Read bytes from a slave
 * @note 0 < @p length <= 255
 * @param i2c The I2Cx peripheral handle
 * @param slaveAddr The slave address (including the R/W bit)
 * @param data The buffer to which the data will be written to
 * @param length The length of the buffer (number of bytes)
 */
uint8_t i2cRead(uint8_t slaveAddr, uint8_t* data, uint8_t length, uint8_t send_stop = 1);

/**
 * @brief Read the register content from a slave (one byte)
 * @param i2c The I2Cx peripheral handle
 * @param slaveAddr The slave address (including the R/W bit)
 * @param regAddr The register which should be read
 * @return The register content
 */
uint8_t i2cReadByte(uint8_t slaveAddr, uint8_t regAddr);

/**
 * @brief Read the register content from a slave (two bytes)
 * @param i2c The I2Cx peripheral handle
 * @param slaveAddr The slave address (including the R/W bit)
 * @param regAddr The register which should be read
 * @return The register content
 */
uint16_t i2cReadWord(uint8_t slaveAddr, uint8_t regAddr);

#endif//STM32_I2C_H
